"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mathViewAndEditPlugin = exports.MathBlockNodeView = exports.MathInlineNodeView = exports.MathNodeView = void 0;
const tslib_1 = require("tslib");
const katex_1 = tslib_1.__importDefault(require("katex"));
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_keymap_1 = require("prosemirror-keymap");
const prosemirror_view_1 = require("prosemirror-view");
const selection_1 = require("../../../utils/selection");
const commands_1 = require("./commands");
const const_1 = require("./const");
const hint_1 = require("./hint");
require("katex/dist/katex.min.css");
class MathNodeView {
    constructor(node, reactRenderer) {
        this.node = node;
        this.reactRenderer = reactRenderer;
        this.texContent = this.getTexContent();
        const elems = this.createDOM();
        this.dom = elems.dom;
        this.contentDOM = elems.contentDOM;
        this.mathViewDOM = elems.mathViewDOM;
        this.mathHintContainerDOM = elems.mathHintContainerDOM;
        this.hintRendererItem = elems.hintRendererItem;
        this.renderKatex();
    }
    update(node) {
        if (this.node.type !== node.type)
            return false;
        this.node = node;
        const newTexContent = this.getTexContent();
        if (this.texContent !== newTexContent) {
            this.texContent = newTexContent;
            this.renderKatex();
        }
        return true;
    }
    ignoreMutation(mutation) {
        return mutation.type === 'childList' && mutation.target === this.mathHintContainerDOM;
    }
    destroy() {
        this.hintRendererItem.remove();
    }
    renderKatex() {
        try {
            katex_1.default.render(this.texContent, this.mathViewDOM, {
                throwOnError: true,
                displayMode: this.isDisplayMode(),
            });
            this.updateErrorView(false);
        }
        catch (err) {
            const errorElem = document.createElement('span');
            errorElem.classList.add('math-error');
            errorElem.innerText = `(error) ${this.texContent}`;
            errorElem.title = String(err);
            this.mathViewDOM.appendChild(errorElem);
            this.updateErrorView(true);
        }
    }
    updateErrorView(isError) {
        this.dom.classList.toggle('math-container-error', isError);
        this.mathViewDOM.classList.toggle('math-view-error', isError);
    }
    getTexContent() {
        return this.node.textContent;
    }
}
exports.MathNodeView = MathNodeView;
class MathInlineNodeView extends MathNodeView {
    destroy() {
        var _a;
        (_a = this.hintRendererItem) === null || _a === void 0 ? void 0 : _a.remove();
    }
    isDisplayMode() {
        return false;
    }
    createDOM() {
        const dom = document.createElement('span');
        dom.classList.add('math-container', 'math-inline-container');
        const mathViewDOM = document.createElement('span');
        mathViewDOM.classList.add('math-view', 'math-inline-view');
        mathViewDOM.contentEditable = 'false';
        const mathInlineDOM = this.createMathInlineDOM();
        const mathHintContainerDOM = document.createElement('div');
        mathHintContainerDOM.contentEditable = 'false';
        mathHintContainerDOM.classList.add((0, hint_1.b)('inline-view'));
        dom.appendChild(mathViewDOM);
        dom.appendChild(mathInlineDOM.container);
        dom.appendChild(mathHintContainerDOM);
        const hintRendererItem = this.reactRenderer.createItem('math-inline-hint', () => (0, hint_1.renderMathHint)({ offset: { left: 3, top: -1 } }, mathHintContainerDOM));
        return {
            dom,
            contentDOM: mathInlineDOM.content,
            mathViewDOM,
            mathHintContainerDOM,
            hintRendererItem,
        };
    }
    // same as math-inline spec toDOM()
    createMathInlineDOM() {
        const container = document.createElement('span');
        container.classList.add(const_1.CLASSNAMES.Inline.Container);
        const sharpBefore = document.createElement('span');
        sharpBefore.classList.add(const_1.CLASSNAMES.Inline.Sharp);
        sharpBefore.contentEditable = 'false';
        sharpBefore.textContent = '$';
        const sharpAfter = sharpBefore.cloneNode(true);
        const content = document.createElement('span');
        content.classList.add(const_1.CLASSNAMES.Inline.Content);
        container.replaceChildren(sharpBefore, content, sharpAfter);
        return { container, content };
    }
}
exports.MathInlineNodeView = MathInlineNodeView;
class MathBlockNodeView extends MathNodeView {
    isDisplayMode() {
        return true;
    }
    createDOM() {
        const dom = document.createElement('div');
        dom.classList.add('math-container', 'math-block-container');
        const mathViewDOM = document.createElement('div');
        mathViewDOM.classList.add('math-view', 'math-block-view');
        mathViewDOM.contentEditable = 'false';
        const contentDOM = document.createElement('div');
        contentDOM.classList.add('math-block');
        const mathHintContainerDOM = document.createElement('div');
        mathHintContainerDOM.contentEditable = 'false';
        mathHintContainerDOM.classList.add((0, hint_1.b)('block-view'));
        dom.appendChild(mathHintContainerDOM);
        dom.appendChild(mathViewDOM);
        dom.appendChild(contentDOM);
        const hintRendererItem = this.reactRenderer.createItem('math-block-hint', () => (0, hint_1.renderMathHint)({ offset: { left: -3 } }, mathHintContainerDOM));
        return { dom, contentDOM, mathViewDOM, mathHintContainerDOM, hintRendererItem };
    }
}
exports.MathBlockNodeView = MathBlockNodeView;
const mathViewAndEditPlugin = ({ reactRenderer }) => new prosemirror_state_1.Plugin({
    props: {
        nodeViews: {
            [const_1.MathNode.Block]: (node) => new MathBlockNodeView(node, reactRenderer),
            [const_1.MathNode.Inline]: (node) => new MathInlineNodeView(node, reactRenderer),
        },
        handleKeyDown: (0, prosemirror_keymap_1.keydownHandler)({
            ArrowLeft: commands_1.moveCursorToEndOfMathInline,
        }),
        decorations: (state) => {
            const { selection } = state;
            if ((0, selection_1.isTextSelection)(selection)) {
                const decorations = [];
                state.doc.nodesBetween(selection.from, selection.to, (node, pos) => {
                    const typeName = node.type.name;
                    if (typeName === const_1.MathNode.Inline || typeName === const_1.MathNode.Block) {
                        decorations.push(prosemirror_view_1.Decoration.node(pos, pos + node.nodeSize, {
                            class: 'math-active',
                        }));
                    }
                });
                if (decorations.length) {
                    return prosemirror_view_1.DecorationSet.create(state.doc, decorations);
                }
            }
            return prosemirror_view_1.DecorationSet.empty;
        },
    },
});
exports.mathViewAndEditPlugin = mathViewAndEditPlugin;
